/*!
 * Copyright (C) 2006-2007 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * \file shoutcast.c
 * \brief SHOUTcast receiver.
 *
 * \verbatim
 *
 * $Log$
 *
 * \endverbatim
 */

#include <cfg/os.h>
#include <cfg/clock.h>
#include <dev/board.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>

#include <sys/version.h>
#include <sys/confnet.h>
#include <sys/atom.h>
#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <sys/socket.h>

#include <hxmp3/mp3dec.h>

#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <pro/dhcp.h>

#include "tlv320dac.h"
#include "config.h"
#include "userif.h"
#include "mp3player.h"
#include "shoutcast.h"

#define MAX_STREAM_TIMEOUTS     5

//#define SKIP_STREAMSTART        (384 * 1024)
#define SKIP_STREAMSTART        0

/*
 * Process embedded meta data.
 */
static int ProcessMetaData(TCPSOCKET * sock, SHOUTCASTINFO * sci)
{
    u_char blks = 0;
    u_short cnt;
    int got;
    int rc = 0;
    u_char to_cnt = 0;
    char *mbuf;
    char *mn1;
    char *mn2;
    char *md1;
    char *md2;

    /*
     * Wait for the lenght byte.
     */
    //while (rip->ri_status == SSTAT_RUNNING) {
    for (;;) {
        if ((got = NutTcpReceive(sock, &blks, 1)) == 1)
            break;
        if (got < 0 || to_cnt++ > MAX_WAITSTREAM) {
            printf("[NoLen]");
            return -1;
        }
    }
    if (blks) {
        if (blks > 32) {
            printf("[Blks=%u]", blks);
            return -1;
        }

        cnt = blks * 16;
        if ((mbuf = malloc(cnt + 1)) == 0) {
            printf("[NoMem]");
            return -1;
        }

        /*
         * Receive the metadata block.
         */
        //while (rip->ri_status == SSTAT_RUNNING) {
        for (;;) {
            if ((got = NutTcpReceive(sock, mbuf + rc, cnt)) < 0) {
                printf("[RxFail]");
                return -1;
            }
            if (got) {
                to_cnt = 0;
                if ((cnt -= got) == 0)
                    break;
                rc += got;
                mbuf[rc] = 0;
            } else if (to_cnt++ > MAX_WAITSTREAM) {
                printf("[RxTo]");
                return -1;
            }
        }

        //ClearMetaData();
        printf("\nMeta='%s'\n", mbuf);
        mn1 = mbuf;
        while (mn1) {
            if ((mn2 = strchr(mn1, ';')) != 0)
                *mn2++ = 0;
            if ((md1 = strchr(mn1, '=')) != 0) {
                *md1++ = 0;
                while (*md1 == ' ' || *md1 == '\'')
                    md1++;
                if ((md2 = strrchr(md1, '\'')) != 0)
                    *md2 = 0;
                if (strcasecmp(mn1, "StreamTitle") == 0) {
                    if (sci->sci_metatitle) {
                        free(sci->sci_metatitle);
                    }
                    sci->sci_metatitle = malloc(strlen(md1) + 1);
                    strcpy(sci->sci_metatitle, md1);
                } else if (strcasecmp(mn1, "StreamUrl") == 0) {
                    if (sci->sci_metaurl) {
                        free(sci->sci_metaurl);
                    }
                    sci->sci_metaurl = malloc(strlen(md1) + 1);
                    strcpy(sci->sci_metaurl, md1);
                }
            }
            mn1 = mn2;
        }
        free(mbuf);
        printf("%lu bytes free\n", NutHeapAvailable());
        //sci->sci_metaupdate = 1;
    }
    return 0;
}

THREAD(ShoutCastThread, arg)
{
    RECEIVERINFO *rip = (RECEIVERINFO *) arg;
    SHOUTCASTINFO *sci = (SHOUTCASTINFO *) rip->ri_bcast;
    STATIONINFO *sip;
    int rbytes;
    int got;
    int tmos;
    int skip_start = SKIP_STREAMSTART;

    for (;;) {
        /* Idle loop. */
        for (;;) {
            printf("[RIDLE]");
            memset(sci, 0, sizeof(SHOUTCASTINFO));
            rip->ri_status = SSTAT_IDLE;
            NutEventBroadcast(&rip->ri_stsevt);

            /* Wait for start event. */
            NutEventWait(&rip->ri_cmdevt, 0);
            printf("[REVT%u]", rip->ri_status);
            if (rip->ri_status == SSTAT_START) {
                break;
            }
        }

        printf("[RRUN]");
        tmos = 0;
        sip = rip->ri_sip;
        sci->sci_metapos = sip->si_metaint;
        /* Broadcast running event. */
        rip->ri_status = SSTAT_RUNNING;
        NutEventBroadcast(&rip->ri_stsevt);

        /* Running loop. */
        while (rip->ri_status == SSTAT_RUNNING) {
            /* Wrap write pointer, if the read pointer is not at the
               buffer start. */
            if (rip->ri_wpos >= MP3_BUFSIZ && rip->ri_rpos > 0) {
                rip->ri_wpos = 0;
            }
            if (rip->ri_wpos >= rip->ri_rpos) {
                /* Write pointer is in front of or equal to the read pointer. 
                   Allow to write up to the end of the buffer. */
                rbytes = MP3_BUFSIZ - rip->ri_wpos;
            } else {
                /* Write pointer is behind read pointer. Allow to write up
                   to the last byte before the read pointer. */
                rbytes = rip->ri_rpos - rip->ri_wpos - 1;
            }
            /* Honor some limit. */
            if (rbytes > 4096) {
                rbytes = 4096;
            }
            /* Do not read beyond meta data. */
            if (sip->si_metaint && rbytes > sci->sci_metapos) {
                rbytes = sci->sci_metapos;
            }

            /* If the buffer is full, wait for a signal from the consumer. */
            if (rbytes == 0) {
//                putchar('!');
                NutEventWait(&rip->ri_wrbque, 1000);
                continue;
            }

            if ((got = NutTcpReceive(sip->si_sock, &rip->ri_buff[rip->ri_wpos], rbytes)) < 0) {
                printf("[RXFAIL(%d)%d]", rbytes, NutTcpError(sip->si_sock));
                break;
            }
            if (got == 0) {
                printf("\n[RTIMEOUT]");
                if (tmos++ > MAX_STREAM_TIMEOUTS) {
                    break;
                }
            } else {
                if (tmos) {
                    tmos--;
                }
                if (skip_start && skip_start > got) {
                    skip_start -= got;
                }
                else {
                    skip_start = 0;
                    rip->ri_wpos += got;
                    rip->ri_avail += got;
                    if (rip->ri_avail >= 2 * MAINBUF_SIZE) {
                        NutEventPost(&rip->ri_rdbque);
                    }
                }
                if (sip->si_metaint) {
                    sci->sci_metapos -= got;
                    if (sci->sci_metapos == 0) {
                        if (ProcessMetaData(sip->si_sock, sci)) {
                            printf("\n[METAFAIL]");
                        }
                        else {
                            DisplayStatus(DIST_FORCE);
                        }
                        sci->sci_metapos = sip->si_metaint;
                    }
                }
            }
        }
    }
}

int ShoutCastInit(RECEIVERINFO * rip)
{
    if (rip->ri_tid == NULL) {
        rip->ri_tid = NutThreadCreate("scast", ShoutCastThread, rip, 1024);
    }
    if (rip->ri_bcast == NULL) {
        rip->ri_bcast = malloc(sizeof(SHOUTCASTINFO));
    }
    if (rip->ri_buff == NULL) {
        rip->ri_buff = malloc(MP3_BUFSIZ + 2 * MAINBUF_SIZE);
    }
    return (rip->ri_tid && rip->ri_bcast && rip->ri_buff) ? 0 : -1;
}
